package com.nexirius.theteam;

import java.net.URL;

import java.io.File;
import java.io.IOException;

import javax.swing.UIManager;

import com.nexirius.util.XString;

import com.nexirius.framework.application.Application;
import com.nexirius.framework.application.DialogManager;
import com.nexirius.framework.onlinehelp.*;
import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.dataeditor.*;
import com.nexirius.framework.layout.*;
import com.nexirius.framework.dataviewer.ViewerCreatorMap;

import com.nexirius.theteam.datacontroller.MainController;
import com.nexirius.theteam.datamodel.*;
import com.nexirius.theteam.layout.*;
import com.nexirius.theteam.persistence.FilePersistence;

//The Team is an application to manage a set of club or team members by saving their
//personal data in a managable persistent list that is stored on the filesystem.
//This is the swing implementation of the application.

public class TheTeam extends Application
    //Variable fields
	static TheTeam onlyInstance = null;
	StringModel statusLineModel = new StringModel("", "StatusLine");
	String installDir;
	//Application Model
	MainModel mainModel = null;
	//Controller
	MainController mainController;
	//Dialol window that display HTML help
	HTMLHelpDialog helpDialog = null;
    //DataModelCommands
	DataModelCommand exitMethod;

	//Constructor gives args to super and sets static field to instance of this
	//so we can check if we are running twice
	public TheTeam(String argv[])
		throws Exception
	{
		super(argv);
		onlyInstance = this;
	}
    //Returns this
	public static TheTeam getApplication()
	{
		return onlyInstance;
	}
    //
	public class MyOnlineHelpManager implements OnlineHelpManager
	{
		public String getHelpString(String resourceKey)
		{
			return getClientResource().getHelpString(resourceKey);
		}

		public void help(String text)
		{
			showHelp(getText(text));
		}
	}
    //Pre-init where we register custom viewer creators and layout classes in the
    //model-viewer map. We create a MainController wich will create an instance of our
    //MainModel and a viewer for the MainModel.
	public void preInit()
	{
		OnlineHelp.init(new MyOnlineHelpManager());
		DialogManager.setErrorAdaptor(this);
		DialogManager.setPopupEditorAdaptor(this);

        //Register viewer creators and layout
		ViewerCreatorMap map = getFactory().getViewerCreatorMap();

		map.register(MainModel.class, new MainLayout(), true);
		map.register(MemberModel.class, new TabbedEditorCreator());
		map.register(MemberModel.class, new MemberLayout(), true);
		map.register(PersonModel.class, new PersonLayout(), true);
		map.register(AddressModel.class, new AddressLayout(), true);
		map.register(PhoneModel.class, new PhoneLayout(), true);
		map.register(PhoneListModel.class, new EditListLayout(0, 15, "PhonesTitle", "PhoneArray", new PhoneArrayLayout(500)), true);
		map.register(AddressListModel.class, new EditListLayout(0, 15, "AddessesTitle", "AddressArray", new AddressArrayLayout(500)), true);

        //Create controller
		try {
			mainController = new MainController(getFactory());
		} catch (Exception e) {
			e.printStackTrace();
		}
        //Set main model field
		mainModel = mainController.getMainModel();
		//Append DataModelCommand instances
		mainModel.appendMethod(exitMethod = new ExitMethod());
		mainModel.appendMethod(new HelpCommand("About", "about.help"));
	}
    //Init method where we init our persistance manager, pull the JComponent out
    //of the DataViewer created by the MainController and add it it to the
    //frames main panel.
	public void init()
	{
		FilePersistence.init("c:\\temp\\theteam");
		getMainPanel().add(mainController.createJComponent());
		getMainFrame().setSize(900, 660);
		DialogManager.center(getMainFrame(), true);
	}
    //Load the main model from the persistance engine.
	public void postInit()
	{
		mainModel.loadAllMembers();
	}
    //Returns application name used for properties files etc.
	public String getApplicationName()
	{
		return "TheTeam";
	}
    //Returns the DataModel used for the frames status line.
	public DataModel getStatusLineModel()
	{
		return statusLineModel;
	}
    //Returns the application main model.
	public DataModel getApplicationModel()
	{
		return mainModel;
	}
    //Method used to set the frames status text.
    //Here we just set the text of our status line string model.
	public void setStatusText(String text)
	{
		statusLineModel.setText(text);
	}
    //Method used to display info for the user
    //Here we just set the status text.
	public void info(String text)
	{
		setStatusText(text);
	}
    //Method used to empty our application main model.
	public void clear()
	{
		mainModel.clear();
	}
    //Returns the location of the help documentation
	public String getHelpBase()
	{
		return getInstallURL() + "help/";
	}
    //Returns the location of the application root dir on the filesystem
	public String getInstallDir()
	{
		if (installDir == null) {
			String prop = "InstallDir";
			installDir = System.getProperty(prop);

			if (installDir == null) {
				System.err.println(prop + " is not set!");

				return "";
			}
		}

		return installDir;
	}
    //Returns the location of the application root dir on the filesystem as a URL
	public String getInstallURL()
	{
		String s = getInstallDir();

		if (s == null) {

			return "";
		}

		XString xs = new XString(s);

		xs.replace(File.separator, "/");

		String path = xs.toString();

		if (!path.startsWith("/")) {
			path = "/" + path;
		}

		if (!path.endsWith("/")) {
			path = path + "/";
		}

		try {
			URL url = new URL("file", "", path);

			return url.toString();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			System.err.println("Can't translate directory into URL:'" + path + "'");
		}

		return "";
	}
    //Returns the application title, wich is the text displayed in the upper left corner
    //of the frame. Here the title is loaded from the applications properties file.
	public String getApplicationTitle()
	{
		String ret = getClientResource().getText("ApplicationTitle");

		return ret;
	}
    //Creates a help dialog with HTML in it that is loaded from the given
    //source path string and displays it on front.
	public void showHelp(String text)
	{
	    //Check if the source path looks alright, otherwise correct it.
		if (!text.startsWith("file") && !text.startsWith("http")) {
			text = getHelpBase() + text;
		}
        //Try to connect to the given URL, otherwise return.
		try {
			URL url = new URL(text);

			try {
				url.openStream();
			}
			catch (IOException ex)
			{
				Object param[] = {text};
                //Display error
				error("NoHelp{0}", param);

				return;
			}
            //Create help dialog and set params
			if (helpDialog == null) {
				helpDialog = new HTMLHelpDialog(getToplevelFrame(), url, getFactory());
				helpDialog.pack();
				helpDialog.setLocation(100, 100);
			}

			helpDialog.setVisible(true);
			helpDialog.toFront();

			helpDialog.help(url);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			Object p[] = new Object[1];

			p[0] = text;
			//Display error
			error("NoHelp{0}", p);
		}
	}
    //Main method that creates an instance of this and starts the application.
	public static void main(String argv[])
	{
		try
		{
			new TheTeam(argv).run();
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
    //
    //DataModelCommand implementations
    //
	class ExitMethod extends DefaultDataModelCommand
	{
		ExitMethod()
		{
		    //Fieldname of this command
			super("Exit");
		}

		public void doAction()
		{
			exit();
		}
	}

	class HelpCommand extends DefaultDataModelCommand
	{
		String _help;

		HelpCommand(String name, String help)
		{
		    //Fieldname of this command
			super(name);
			_help = help;
		}

		public void doAction()
		{
			OnlineHelp.help(_help);
		}
	}
}

