package com.nexirius.theteam.datamodel;

import com.nexirius.framework.datamodel.SimpleModel;
import com.nexirius.framework.datamodel.StringModel;
import com.nexirius.framework.datamodel.DataModelVector;
import com.nexirius.framework.datamodel.DataModelListener;
import com.nexirius.framework.datamodel.DataModelEvent;
import com.nexirius.framework.datamodel.ModelFlag;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.util.CopyPairs;

//
//Model used to manage a full name that is composed of several strings.
//Datamodels storing parts of the fulle name(should be a string containing model)
//are added or removed to a vector  wich is then calculated, that is all
//components of the vector a aligned in a StringBuffer seperated by a space.
//The content of the buffer is then read into the our StringModel. (setText())
//

public class ConcatModel extends StringModel implements DataModelListener
{
    //name vector
	DataModelVector summanden = new DataModelVector();
	//space
	String separator = " ";

    //Constructor
	public ConcatModel(String name)
	{
		super("", name);
		setTransient(true);
	}

    //Adds the given data model to our vector and calls the calculate method
	public void addSummand(SimpleModel summand)
	{
		summanden.append(summand);
		summand.addDataModelListener(this);
		calculate();
	}

	//Allows to set a custom seperator for the space between the components of our name vector.
	public void setSeparator(String s)
	{
		separator = s;
	}

    //Removes a component from the name vector and calls calculate
	public void removeSummand(SimpleModel summand)
	{
		summanden.removeItem(summand);
		summand.removeDataModelListener(this);
		calculate();
	}

    //Reads the vector components and writes them into a StringBuffer seperating
    //them by the seperator string and stores the result in the text value of this StringModel.
	public void calculate()
	{
		StringBuffer newValue = new StringBuffer();
		boolean start = true;

		for (SimpleModel m = (SimpleModel)summanden.firstItem(); m != null; m = (SimpleModel)summanden.nextItem()) {
			if (start) {
				start = false;
			} else {
				newValue.append(separator);
			}

			if (!m.isEmpty()) {
				newValue.append(m.getText());
			}
		}
        //Set our value to the result contained by our string buffer.
		setText(newValue.toString());
	}
    //
    //DataModelListener for the components of the name vector
    //
	public void dataModelChangeValue(DataModelEvent event)
	{
		calculate();
	}

	public void dataModelChangeStructure(DataModelEvent event)
	{
	}

	public void dataModelGrabFocus(DataModelEvent event)
	{
	}

    //Duplication method
	public synchronized DataModel duplicate(DataModel instance, CopyPairs copyPairs)
	{
		if (instance == null) {
			instance = new ConcatModel(getFieldName());
		}

		return super.duplicate(instance, copyPairs);
	}
}

