package com.nexirius.theteam.datamodel;

import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.application.DialogManager;

//Abstract model that stores a list as an array and has some commands to edit the array

public abstract class EditListModel extends StructModel
{
    //Array and commands
	ArrayModel list;
	public DataModelCommand add;
	public DataModelCommand edit;
	public DataModelCommand delete;

	//Constructor with fieldname as argument
	public EditListModel(String name)
	{
		super(name);
	}
    //Constructor with fieldname and array as argument
	public EditListModel(String name, ArrayModel list)
	{
		super(name);
		init(list);
	}
    //Returns list array
	public ArrayModel getList()
	{
		return list;
	}
    //Returns size of array
	public int getSize()
	{
		return list.getSize();
	}
    //Returns name of add command
	public String getAddItemCommandName()
	{
		return "AddItem";
	}
    //Returns name of edit command
	public String getEditItemCommandName()
	{
		return "EditItem";
	}
    //Returns name of delete command
	public String getDeleteItemCommandName()
	{
		return "DeleteItem";
	}
    //init method, needs a list array as argument. appends array and commands.
    //adds a focus listener to the list.
	private void init(ArrayModel list)
	{
		this.list = list;
		append(list);

		list.addDataModelListener(new MyFocusListener());

		appendMethod(add = new AddItemCommand(getAddItemCommandName()));
		appendMethod(edit = new EditItemCommand(getEditItemCommandName()));
		appendMethod(delete = new DeleteItemCommand(getDeleteItemCommandName()));
	}
    //Focus listener for list array
	class MyFocusListener extends DataModelAdaptor
	{
		public void dataModelGrabFocus(DataModelEvent event)
		{
			try {
				event.getTransmitterVector().lastItem();
				DataModel dm = event.getTransmitterVector().previousItem();

				editItem(dm);
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
				System.out.println(ex);
			}
		}
	}
	//Returns the highlighted item in the list
	public DataModel getHighlightedItem()
	{
		return getList().getHighlightedItem();
	}
    //
    //Methods called by datamodel commands (not implemented here because of lack of a concrete context)
    //
	public DataModel addItem()
	{
		return null;
	}
	
	public void editItem(DataModel item)
	{
	}

	public void deleteItem(DataModel item)
	{
	}
    //
    //DataModelCommand implementations for this model
    //
    //calls the delete method after asking the user to confirm    
	class DeleteItemCommand extends DefaultDataModelCommand
	{
		DeleteItemCommand(String name)
		{
			super(name);
		}
		public void doAction()
		{
			DataModel highlighted = getHighlightedItem();

			if (highlighted != null) {

				if (DialogManager.ask("ReallyDeleteHighlightedItem", "Yes", "Cancel", false)) {

					deleteItem(highlighted);
				}
			} else {
				DialogManager.error("NothingHighlighted");
			}
		}
	}
    //calls the edit method
	class EditItemCommand extends DefaultDataModelCommand
	{
		EditItemCommand(String name)
		{
			super(name);
		}
		public void doAction()
		{
			DataModel highlighted = getHighlightedItem();

			if (highlighted != null) {
				editItem(highlighted);
			} else {
				DialogManager.error("NothingHighlighted");
			}
		}
	}
    //calls the add item method and sorts the list afterwards
	class AddItemCommand extends DefaultDataModelCommand
	{
		AddItemCommand(String name)
		{
			super(name);
		}

		public void doAction()
		{
			DataModel item = addItem();
			
			if (item != null) {
				list.sortInsert(item);
			}
		}
	}
}

