package com.nexirius.theteam.datamodel;

import com.nexirius.framework.datamodel.*;
import com.nexirius.theteam.persistence.TeamPersistence;


//
//The Team application main model
//
public class MainModel extends StructModel
{
    //Child model fields
	MemberListModel memberList;
	MemberModel selected;
	AddressModel address;

	//Constructor
	public MainModel()
	{
	    //Fieldname
		super("Main");
		init();
	}
    //Init method creates child models and appends them to main model (this).
    //We add a listener to the list model.
	private void init()
	{
		append(memberList = new MemberListModel());
		append(selected = new MemberModel());
		append(address = new AddressModel());
		
		memberList.addDataModelListener(new MyListener());
	}
	//Method to fill up child array from file
	public void loadAllMembers() 
	{
		TeamPersistence.getAllMembers(memberList.getList());
	}
    //Returns child list model
	public MemberListModel getMemberList()
	{
		return memberList;
	}
    //Method to update the addresses of the selected list item and drop
    //the main address data into our child model.
	public void updateSelected()
	{
		try {
			MemberModel m = (MemberModel)memberList.getHighlightedItem();

			if (m == null) {
				selected.clear();
				
				return;
			}
            //Drop the model data of the selected item into our child model.
			selected.dropData(m.dragData());
			selected.getPerson().dropData(m.getPerson().dragData());
			//Drop the adresses from file into our MemberModel
			TeamPersistence.getAddressesForMember(selected);

            //Members main address
			AddressModel mainAddress = selected.getMainAddress();

            //Drop the members main address data into our child address model
			if (mainAddress == null) {
				address.setFlag(ModelFlag.INVISIBLE, true);
			} else {
				address.setFlag(ModelFlag.INVISIBLE, false);
				address.dropData(mainAddress.dragData());
			}
		}
		catch (Exception ex) {
			//selected.setFlag(ModelFlag.INVISIBLE, true);
			ex.printStackTrace();
		}
		//selected.setFlag(ModelFlag.INVISIBLE, false);
	}
    //Listener for the list model.
    //In case of value change or highlight change, we call updateSelected()
	class MyListener extends DataModelAdaptor
	{
		public void dataModelChangeValue(DataModelEvent event)
		{
			updateSelected();
		}

		public void dataModelEdit(DataModelEvent event)
		{
			if (event.getId() == DataModelEvent.HIGHLIGHT_CHANGED) {
				updateSelected();
			}
		}
	}
}
