package com.nexirius.theteam.datamodel;

import com.nexirius.theteam.persistence.DBObjectModel;

import com.nexirius.framework.datamodel.*;

//
//Model stores complete member information by appending child models that store
//person, address and phone number information. Additionally ID and type is stored
//in a StringModel and a ComboBoxModel.
//
public class MemberModel extends DBObjectModel
{
    //type combobox arrays
	public static final String TYPE_LABELS[] = {"active", "passive", "donator"};
	static SimpleArrayModel typeLabels = new SimpleArrayModel(TYPE_LABELS, "type_labels");

	//child model fields
	StringModel personId;
	PersonModel person;
	AddressListModel addresses;
	PhoneListModel phones;
	ComboBoxModel type;
    //Constructor
	public MemberModel()
	{
		super("Member"); //default fieldname
		init();
	}
    //create and append child models
	private void init()
	{
		append(personId = new StringModel("", "personId"));
		append(person = new PersonModel());
		append(addresses = new AddressListModel());
		append(phones = new PhoneListModel());
		append(type = new ComboBoxModel(0, typeLabels, "type"));
		
		person.setTransient(true);
		addresses.setTransient(true);
		phones.setTransient(true);
	}

    //Returns the first address in the list or null
	public AddressModel getMainAddress()
	{
		if (addresses.getList().getSize() > 0) {
			return (AddressModel)addresses.getList().getDataModel(0);
		}

		return null;
	}
    //Return the ArrayModel that stores the addresses in the address list model
	public AddressArrayModel getAddresses() 
	{
		return (AddressArrayModel)addresses.getList();
	}
    //Returns the person model
	public PersonModel getPerson() {
		return person;
	}
    //Returns the phone number array
	public PhoneArrayModel getPhones() {
		return (PhoneArrayModel)phones.getList();
	}
	//Sets the person ID 
	public void setPersonId(String i)
	{
		personId.setText(i);
	}
    //Returns person ID
	public String getPersonId()
	{
		return personId.getText();
	}
}

