package com.nexirius.theteam.datamodel;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import com.nexirius.util.CopyPairs;
import com.nexirius.framework.datamodel.DataModelEditListener;
import com.nexirius.framework.datamodel.DataModelEvent;
import com.nexirius.framework.datamodel.DataModel;
import com.nexirius.framework.datamodel.DateModel;
import com.nexirius.framework.datamodel.ModelFlag;

//
//Model subclassing the DateModel to store a date in the DD.MM.YYYY way.
//

public class StandardDateModel extends DateModel
{
    //Our date format
	public static DateFormat DATE_FORMAT = new SimpleDateFormat("dd.mm.yyyy");

	//Constructor with name and date argument
	public StandardDateModel(Date date, String name)
	{
		super(date, name);
		init();
	}

	//Constructor with name argument only.
	//NULL_TIME is set as date.
	public StandardDateModel(String name)
	{
		super(new Date(NULL_TIME), name);
		init();
		setFlag(ModelFlag.EMPTY, true);
	}

	//set the proper date format
	private void init()
	{
		setDateFormat(DATE_FORMAT);
	}
    //fires value change event. we set the EMPTY flag correctly and call
    //the overridden super method.
	public void fireValueChange()
	{
		setFlag(ModelFlag.EMPTY, isNull());

		super.fireValueChange();
	}
    //Returns true when date model is not empty
	public boolean filled()
	{
		return !isNull();
	}
    //Duplication method
	public synchronized DataModel duplicate(DataModel instance, CopyPairs copyPairs)
	{
		if (instance == null) {
			instance = new StandardDateModel(getDate(), getFieldName());
		}

		return super.duplicate(instance, copyPairs);
	}
}

