package com.nexirius.theteam.layout;

import java.awt.*;

import com.nexirius.framework.command.*;
import com.nexirius.framework.datamodel.*;
import com.nexirius.framework.dataeditor.*;
import com.nexirius.framework.dataviewer.*;
import com.nexirius.framework.layout.*;

public class StandardLayout extends StructureLayoutItem
{
	public double totalWidth = 65;
	public double totalHeight = 45;

	public double wBigFrame = totalWidth - 5;

	protected int x = 0;
	protected int y = 0;
	protected int wcell = 10;
	protected int hcell = 8;
	protected int gap = 3;

	public double wframe = 40;
	public double wdialog = wframe - 2;
	public double wlabel = 16;
	public double wfield = wdialog - wlabel;
	public double wbutton = wlabel;
	public double hfield = 2;
	public double hlabel = 2;
	public double hbutton = hfield;

	protected static boolean initDone = false;
	protected static MultiplePropertyEntry labelProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry bannerProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry fieldProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry fieldNoEditProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry buttonProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry numberProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry numberNoEditProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry bemerkungProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry frageProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry titleProps = new MultiplePropertyEntry();
	protected static MultiplePropertyEntry blackTitleProps = new MultiplePropertyEntry();
	protected static SimpleViewerCreator svc = new SimpleViewerCreator();

	public StandardLayout()
	{
		initProperties();
	}

	public void setBigFrame()
	{
		setWFrame(wBigFrame);
	}

	public void setWFrame(double w)
	{
		wframe = w;
		wdialog = wframe - 2;
		wfield = wdialog / 1.5;

		if (wfield > 25) {
			wfield = 25;
		}

		wlabel = wdialog - wfield;
		wbutton = wlabel;
	}

	public void appendLabel(String name)
	{
		append(new LabelLayoutItem(name, x, y, w(wlabel), h(hlabel), labelProps));
		right(wlabel);
	}

	public void appendBanner(String name)
	{
		append(new LabelLayoutItem(name, x, y, w(wlabel), h(hlabel), bannerProps));
		right(wlabel);
	}

	public void appendTextField(String name)
	{
		appendLabel(name);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), null, null, false, null, fieldProps));
		left(wlabel);
		down(hfield);
	}

	public void appendMultilineTextField(String name, double lines)
	{
		appendLabel(name);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(lines * hfield), new TextAreaEditorCreator(), null, false, null, fieldProps));
		left(wlabel);
		down(lines * hfield);
	}

	public void appendTextField(String name, String reskey)
	{
		appendLabel(reskey);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), null, null, false, reskey, fieldProps));
		left(wlabel);
		down(hfield);
	}

	public void appendBooleanField(String name)
	{
		right(wlabel);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), null, null, false, null, fieldProps));
		left(wlabel);
		down(hfield);
	}

	public void appendSplitField(String name1, String name2, String label, double split)
	{
		double wfield1 = wfield * split;
		double wfield2 = wfield - wfield1;

		appendLabel(label);
		append(new StructureFieldLayout(name1, x, y, w(wfield1), h(hfield), null, null, false, null, fieldProps));
		right(wfield1);
		append(new StructureFieldLayout(name2, x, y, w(wfield2), h(hfield), null, null, false, null, fieldProps));
		left(wfield1);
		left(wlabel);
		down(hfield);
	}

	public void appendNumberSplitField(String name1, String name2, String label, double split)
	{
		double wfield1 = wfield * split;
		double wfield2 = wfield - wfield1;

		appendLabel(label);
		append(new StructureFieldLayout(name1, x, y, w(wfield1), h(hfield), null, null, false, null, numberProps));
		right(wfield1);
		append(new StructureFieldLayout(name2, x, y, w(wfield2), h(hfield), null, null, false, null, numberProps));
		left(wfield1);
		left(wlabel);
		down(hfield);
	}

	public void appendNoEditSplitField(String name1, String name2, String label, double split)
	{
		double wfield1 = wfield * split;
		double wfield2 = wfield - wfield1;

		appendLabel(label);
		append(new StructureFieldLayout(name1, x, y, w(wfield1), h(hfield), svc, null, false, null, numberNoEditProps));
		right(wfield1);
		append(new StructureFieldLayout(name2, x, y, w(wfield2), h(hfield), svc, null, false, null, numberNoEditProps));
		left(wfield1);
		left(wlabel);
		down(hfield);
	}

	public void appendFrame(String title, double elements)
	{
		append(new TitledBorderLayoutItem(title, x, y, w(wframe), h(elements * hfield + hfield + 1)));
		down(hfield);
		right(1);
	}

	public void appendStaticText(String text, double lines)
	{
		append(new StaticTextFieldLayoutItem(text, x, y, w(wframe), h(lines * hfield), bemerkungProps));
		down(lines * hfield);
	}

	public void appendStaticText(String text, double lines, double width)
	{
		append(new StaticTextFieldLayoutItem(text, x, y, w(width), h(lines * hfield), bemerkungProps));
		down(lines * hfield);
	}

	public void endFrame()
	{
		down(1);
		left(1);
	}

	public void appendNumberField(String name, String fieldname)
	{
		appendLabel(name);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), null, null, false, fieldname, numberProps));
		left(wlabel);
		down(hfield);
	}

	public void appendNumberField(String name)
	{
		appendNumberField(name, null);
	}

	public void appendOutputField(String name)
	{
		appendLabel(name);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), svc, null, false, null, numberNoEditProps));
		left(wlabel);
		down(hfield);
	}

	public void appendOutputTextField(String name)
	{
		appendLabel(name);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), svc, null, false, null, fieldNoEditProps));
		left(wlabel);
		down(hfield);
	}

	public void appendTextFieldRight(String name)
	{
		appendLabel(name);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hfield), null, null, false, null, fieldProps));
		right(wfield);
	}

	public void appendField(String name, double width, double height)
	{
		append(new StructureFieldLayout(name, x, y, w(width), h(height), null, null, false, null, null));
		down(height);
	}

	public void appendFieldRight(String name, double width, double height)
	{
		append(new StructureFieldLayout(name, x, y, w(width), h(height), null, null, false, null, null));
		right(width);
	}

	public void appendPopupEditor(String name, double width, double height)
	{
		append(new StructureFieldLayout(name, x, y, w(width), h(height), new PopupEditorCreator(), null, false, null, null));
		down(height);
	}

	public void appendButtonRight(String name, double width)
	{
		append(new ButtonLayoutItem(name, x, y, w(width), h(hbutton), null, buttonProps));
		right(width);
	}

	public void appendButtonDown(String name)
	{
		append(new ButtonLayoutItem(name, x, y, w(wbutton), h(hbutton), null, buttonProps));
		down(hbutton);
	}


	public void appendBooleanRadioField(String name)
	{
		double hh = hfield;

		append(new LabelLayoutItem(name, x, y, w(wlabel), h(hlabel), labelProps));
		right(wlabel);
		append(new StructureFieldLayout(name, x, y, w(wfield), h(hh), new RadioBoxViewerCreator(), null, false, null, null));
		left(wfield);
		left(wlabel);
		down(hh);
	}

	public void defaultSize()
	{
		setSize(w(wframe), y);
	}

	private static void initProperties()
	{
		if (initDone) {

			return;
		}

		initDone = true;
		
		//labelProps.append(new SinglePropertyEntry("Font", "Arial-12"));
		labelProps.append(new SinglePropertyEntry("Foreground", "labelForeground"));
		labelProps.append(new SinglePropertyEntry("Background", "labelBackground"));
		labelProps.append(new SinglePropertyEntry("Alignment", "-1"));

		//bannerProps.append(new SinglePropertyEntry("Font", "Arial-12"));
		bannerProps.append(new SinglePropertyEntry("Foreground", "labelForeground"));
		bannerProps.append(new SinglePropertyEntry("Background", "labelBackground"));
		bannerProps.append(new SinglePropertyEntry("Alignment", "0"));

		//fieldProps.append(new SinglePropertyEntry("Font", "Arial-12"));
		fieldProps.append(new SinglePropertyEntry("Foreground", "fieldForeground"));
		fieldProps.append(new SinglePropertyEntry("Background", "fieldBackground"));
		fieldProps.append(new SinglePropertyEntry("Alignment", "-1"));

		//fieldNoEditProps.append(new SinglePropertyEntry("Font", "Arial-12"));
		fieldNoEditProps.append(new SinglePropertyEntry("Foreground", "noeditForeground"));
		fieldNoEditProps.append(new SinglePropertyEntry("Background", "noeditBackground"));
		fieldNoEditProps.append(new SinglePropertyEntry("Alignment", "-1"));

		//buttonProps.append(new SinglePropertyEntry("Font", "Arial-10"));
		buttonProps.append(new SinglePropertyEntry("Foreground", "buttonForeground"));
		buttonProps.append(new SinglePropertyEntry("Background", "buttonBackground"));
		buttonProps.append(new SinglePropertyEntry("Alignment", "0"));

		//numberProps.append(new SinglePropertyEntry("Font", "Courier-12"));
		numberProps.append(new SinglePropertyEntry("Foreground", "numberForeground"));
		numberProps.append(new SinglePropertyEntry("Background", "numberBackground"));
		numberProps.append(new SinglePropertyEntry("Alignment", "1"));

		//numberNoEditProps.append(new SinglePropertyEntry("Font", "Courier-12"));
		numberNoEditProps.append(new SinglePropertyEntry("Foreground", "noeditForeground"));
		numberNoEditProps.append(new SinglePropertyEntry("Background", "noeditBackground"));
		numberNoEditProps.append(new SinglePropertyEntry("Alignment", "1"));

		//bemerkungProps.append(new SinglePropertyEntry("Font", "Arial-ITALIC-12"));
		bemerkungProps.append(new SinglePropertyEntry("Foreground", "bemerkungForeground"));
		bemerkungProps.append(new SinglePropertyEntry("Background", "bemerkungBackground"));
//		bemerkungProps.append(new SinglePropertyEntry("Alignment", "0"));

		//frageProps.append(new SinglePropertyEntry("Font", "Arial-12"));
		frageProps.append(new SinglePropertyEntry("Foreground", "frageForeground"));
		frageProps.append(new SinglePropertyEntry("Background", "frageBackground"));

		//titleProps.append(new SinglePropertyEntry("Font", "Arial-BOLD-12"));
		titleProps.append(new SinglePropertyEntry("Foreground", "titleForeground"));
		titleProps.append(new SinglePropertyEntry("Background", "titleBackground"));

		//blackTitleProps.append(new SinglePropertyEntry("Font", "Arial-BOLD-12"));
		blackTitleProps.append(new SinglePropertyEntry("Foreground", "white"));
		blackTitleProps.append(new SinglePropertyEntry("Background", "black"));
	}

	public void center(double w)
	{
		pos((totalWidth - w) / 2.0, 1.0);
	}

	public void pos(double u, double v)
	{
		x = (int)(u * (gap + wcell));
		y = (int)(v * (gap + hcell));
	}

	public int w(double v)
	{
		return (int)(v * (wcell + gap)) - gap;
	}

	public int h(double v)
	{
		return (int)(v * (hcell + gap)) - gap;
	}

	public void right(double v)
	{
		x += (int)(v * (wcell + gap));
	}

	public void left(double v)
	{
		x -= (int)(v * (wcell + gap));
	}

	public void down(double v)
	{
		y += (int)(v * (hcell + gap));
	}

	public void up(double v)
	{
		y -= (int)(v * (hcell + gap));
	}
}
