package com.nexirius.theteam.persistence;

import com.nexirius.framework.datamodel.*;

//
//Implementation of DBObject using a StructModel. DataModels are subclassing this
//implementation for persistancy.
//

public class DBObjectModel extends StructModel implements DBObject
{
    //Child model fields for ID & version
	public StringModel id;
	public StringModel version;

	//Constructor with no arguments just setting a default field name
	public DBObjectModel() 
	{
		this("DBObject");
	}
    //Constructor with field name as argument setting name and appending
    //child models to this.
	public DBObjectModel(String fieldName) 
	{
		super(fieldName);
		append(id = new StringModel("", FIELD_ID));
		append(version = new StringModel("", FIELD_VERSION));
	}
	//
    //Public setter/getter for ID & version
    //
  	public void setID(String id)
	{
		this.id.setText(id);
	}

	public String getID()
	{
		return id.getText();
	}

	public void setVersion(String version) 
	{
		this.version.setText(version);
	}

	public String getVersion() 
	{
		return version.getText();
	}

	//Comparasion method implementation
	/**
	 * use a long integer to represent version
	 */
	public int compareVersion(String version) 
	{
		long thisVersion = Long.parseLong(getVersion());
		long otherVersion = Long.parseLong(version);
		
		if (thisVersion > otherVersion) {

			return 1;
		}
		
		if (thisVersion < otherVersion) {
			
			return -1;
		}

		return 0;
	}
    //Returns child model strings by fieldname
	public String getProperty(String propertyName) 
	{
		return getChildText(propertyName);
	}
}
